/**********************************************************************

This file is part of the Quantum Computation Language QCL.

(c) Copyright by Bernhard Oemer <oemer@tph.tuwien.ac.at>, 1998

This program comes without any warranty; without even the implied 
warranty of merchantability or fitness for any particular purpose.

     This program is free software under the terms of the 
     GNU General Public Licence (GPL) version 2 or higher

************************************************************************/


#include "syntax.h"
#include "symbols.h"
#include "error.h"
#include "yacc.h"
#include "quheap.h"
#include "parse.h"
#include "options.h"
#include "debug.h"
#include "debug.h"

objlist::objlist() : list<sObject*>() {
  sObject *p;
  while(p=parseobj()) push_back(p);
  yyCleanUp();
}

objlist::~objlist() {
  while(size()) {
    if(front()->isStmt()) delete front();
    pop_front();
  }
}

int shell_depth=-1;

int qcllist(SymTable *loc,SymTable *gl,QuHeap *qh,sDef *def,tError *err,int flags) {
  objlist o;
  sObject *p;
  int r;

  while(o.size()) {
    p=o.front();
    if(optEcho) err->print(p->prtstr(INDENT));
    err->reset();
    if(p->isInclude()) {
      r=qclfile(((sInclude*)p)->filename(),loc,gl,qh,0,err,flags|QL_ERRORRETURN);
      if(r) return r;
      goto qlcontinue;
    }
    r=p->typecheck(loc,gl,0,err);
    if(r && p->isDef() && err->errtype()==errSCOPE && (flags&QL_IGNOREDEFERR)) 
      goto qlcontinue;
    if(!r && p->isDef()) {
      r=((sDef*)p)->define(loc,gl,qh,0,err);
      if(!r) {
        o.pop_front();
        continue;
      }
    }
    if(!r && p->isStmt()) {
      r=((sStmt*)p)->exec(loc,gl,qh,0,err);
    }
    if(r) {
      if(err->isExit()) {
        if(flags&QL_IGNOREEXIT) goto qlcontinue;
        return r;
      }
      if(flags&QL_ERRORABORT) qclabort(err->errmsg());
      if(!isErrorReported && !(flags&QL_NOERRMSG)) qclerror(err->errmsg());
      if(flags&QL_ERRORRETURN) return r;
    };
qlcontinue:
    delete p;
    o.pop_front();
  }
  return 0;
}      

int qclfile(string fname,SymTable *loc,SymTable *gl,QuHeap *qh,sDef *def,tError *err,int flags) { 
  FILE *f;
  string s;
  int i,r;

  f=openqclfile(fname);  
  if(!f) { qclerror("Can't open "+fname); return 1; }
  if(chksyntax(f)) return 1; 
  f=openqclfile(fname);
  if(!f) { 
    qclerror("Can't reopen "+fname);
    return 1;
  }
  yyScanFile(f);
  r=qcllist(loc,gl,qh,0,err,flags | QL_ERRORRETURN | (optAllowRedefines ? QL_IGNOREDEFERR : 0));
  return r; 
}  

void qclshell(SymTable *loc,SymTable *gl,QuHeap *qh,sDef *def,tError *err,int cond) { 
  sObject *p;
  string s="",prompt="qcl> ";
  int r;
  sRoutDef *d=(sRoutDef*)def;
  
  SymTab l,g;
  SymTabComb lcomb((SymTable*)&l,loc),gcomb((SymTable*)&g,gl);
  QuHeap lqh(qh,0);
  tError e;

  shell_depth++;
  if(cond==DB_ESCAPE) {
    s="shell escape";
    if(def) s+=" in "+d->type().str()+" "+d->id()+"("+d->args()->prtstr()+")";
    if(yyToplevelFile) s+=" ignored (file "+yyFilename+").";
    s+="\n";
    err->print(s);
  }
  if(cond==DB_ERROR) {
    qclerror(err->errmsg());
  }
  if(d && d->isQuDef()) {
    cout.form("Bennet-flag: %d, Delay-flag: %d\n",
               d->isBennet(),qh->isDelayed());
  }
  prompt="qcl";
  if(shell_depth>=1) prompt+=(char)('0'+shell_depth);
  prompt+=cond==DB_ERROR ? "$ " : "> ";
    
  while(1) {
    if(optAutoDump && isStateModified) {
      term_format=TERM_FORMAT_ALGEBRAIC;
      bitvec_format=BITVEC_FORMAT_BIN|BITVEC_FORMAT_KET;
      if(optDumpFormat=='a' && qh->nBits()>32 || optDumpFormat=='x') 
        bitvec_format|=BITVEC_FORMAT_HEX;
      s=string("[")+dec(lqh.nBits()-lqh.nFree())+"/"+dec(lqh.nBits())+"] ";
#ifdef __STRING__
      printbase(cout,*lqh.state(),s.c_str(),"\n");
#else
      printbase(cout,*lqh.state(),s,"\n");
#endif
    }
    isStateModified=0;
    isErrorReported=0;
    s=qclinput(prompt);
    if(s=="\n") continue;
    if(s=="") { cout << "\n"; break; }
    s+=";";
    if(chksyntax(s)) {
      if(!isErrorReported) qclerror("syntax error");
      continue;
    }
    yyScanString(s);
    r=qcllist(&lcomb,&gcomb,&lqh,def,&e,QL_ERRORRETURN);
    if(r && e.isExit()) break;
  };
  shell_depth--;
  isErrorReported=1;
  return;
}
    
